//------------------------------------------------------------------------
//
// Class:     CDummyCCD
// Filename:  DummyCCD.h
// Purpose:   Class definition for sample "DummyCCD" MaxIm CCD plug-in
// Version:   1.0
//
// Copyright (c) 1998-2003 Diffraction Limited
// Written by Douglas B. George
//
//------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////
// USE OF THIS SOFTWARE
//
// This source code is licensed for use in developing a plug-in DLL for controlling 
// CCD cameras under MaxIm CCD only.  This plug-in source code file shall not be 
// distributed in whole or in part without the express written permission of 
// Diffraction Limited,  Executable DLL files built using this source code 
// may be freely distributed.  

#if !defined(AFX_DUMMYCCD_H__FF369648_87BA_11D2_AA9C_444553540000__INCLUDED_)
#define AFX_DUMMYCCD_H__FF369648_87BA_11D2_AA9C_444553540000__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

// Defined for dummy camera
enum GuiderErrorMode {
	GE_OFF,
	GE_RANDOM,
	GE_SINUSOIDAL,
	GE_BOTH,
};

#include "MaxImCCDPlugIn.h"

class CDummyCCD : public CMaxImCCDPlugIn  
{
protected:
	int BinningX;					// Binning factor
	int BinningY;
	clock_t StopTime;				// CPU clock time for simulated end of exposure
	unsigned short *Buffer;			// Buffer for readout
	int SubframeXSize;				// Subframe dimensions
	int SubframeYSize;
	int SubframeXStart;
	int SubframeYStart;
	PBOOL Shutter;					// Shutter open if true
	double Temperature;				// Temperature of CCD chip
	double TSetPoint;				// Setpoint 
	PBOOL Exposing;					// Currently exposing if true
	PBOOL Reading;					// Currently reading if true
	int Pixel;						// Pixel counter for simulating readout
	PBOOL LightFrame;				// Indicates if a light frame (for simulated data)
	PBOOL NoiseOn;					// Simulates image noise floor
	GuiderErrorMode GuiderErrors;	// Simulates guider errors
	PBOOL FilterDelayOn;				// Simulates delay rotating filter wheel
	double OffX;					// Used for guider error simulation
	double OffY;					// Used for guider error simulation

public:
	CDummyCCD();
	virtual ~CDummyCCD();

	// GetParameters tells MaxIm CCD what information to ask the user for
	void GetParameters ( 
		DialogContents &Contents,				// Controls the setup dialog box
		PBOOL &HasCamera,						// If true, MaxIm CCD will add this to camera and guider lists
		PBOOL &HasFilterWheel,					// If true, MaxIm CCD will add camera's internal filter wheel to list
		PBOOL &HasGuiderRelays					// If true; MaxIm CCD will use camera's autoguider relays
		);

	// OpenCamera initializes the link to the camera and allocates buffer memory
	int OpenCamera(
		const char * FilePath,					// File path provided by user (if UseFilePath above is true)
		int Param[ NUM_PARAM ]					// 0-based index for each user parameter input
		);

	// CloseCamera shuts down the link to the camera and deallocates buffer memory
	void CloseCamera();

	// GetArraySize returns the number of pixels in X and Y for the CCD array (unbinned)
	void GetArraySize ( 
		int &XSize,								// Number of array columns (X)
		int &YSize								// Number of array rows (Y)
		);

	// GetImageSize returns the number of pixels in each dimension for the current image, 
	// plus offset value for subframe; includes binning
	void GetImageSize ( 
		int &XStart,							// Column offset for subframe (X) (after binning)
		int &YStart,							// Row offset for subframe (Y) (after binning)
		int &XSize,								// Number of image columns (X)
		int &YSize								// Number of image rows (Y)
		);

	// GetPixelAspect returns the size of the pixels in microns (micrometers)
	PBOOL GetPixelAspect ( 
		double &PixX,							// Pixel physical X dimension in microns
		double &PixY, 							// Pixel physical Y dimension in microns
		PBOOL IncludeBinning						// If TRUE, multiply above by binning factors
		);

	// GetImageBuffer returns a pointer to the image buffer allocated by OpenCamera and filled by TransferImage
	unsigned short * GetImageBuffer();

	// StartExposures starts a CCD exposure
	int StartExposure ( 
		unsigned int Exposure,					// Exposure duration in 1/100ths of a second
		int XStart,								// X Offset to start of subframe 
		int YStart, 							// Y Offset to start of subframe 
		int NumX, 								// Number of columns for subframe
		int NumY, 								// Number of rows for subframe
		int BinX, 								// X binning factor (all above numbers are in binned pixel units)
		int BinY, 								// Y binning factor
		PBOOL Light, 							// If true open shutter for exposure, if false close shutter
		PBOOL FastReadout, 						// If true we are in focus mode (optionally use faster readout mode)
		PBOOL HoldShutterOpen 					// If true we are in guider mode (optionally leave shutter open for readout)
		);

	// AbortExposure aborts the current imaging operation and returns the camera to idle
	// (n.b. If camera hardware needs to be reset, GetCameraState should return CS_FLUSHING until cleared)
	void AbortExposure();

	// TransferImage downloads the image data from the camera
	int TransferImage ( 
		PBOOL &TransferDone,						// May return periodically to update GUI, true when done
		int &PercentDone						// If returns periodically during download, percentage complete
		);

	// SetTemperature controls the CCD cooler
	PBOOL SetTemperature ( 
		PBOOL ControlOn,							// If true, controller is turned on, if FALSE, turned off
		PBOOL GoToAmbient,						// GoToAmbient should start a gradual warm-up to ambient
		double Temp								// Desired setpoint when ControlOn = true and GoToAmbient = FALSE
		);

	int GetTemperature ( 
		CoolerState &State,						// Current temperature controller status
		double &Temp,							// Temperature of CCD chip (FLT_MAX if not available)
		double &TempAmbient,					// Ambient temperature if available (FLT_MAX if not)
		unsigned short &Power					// Power level in percent (0-100), return > 100 if not available
		);

	// GetCameraStats returns various information about the camera
	void GetCameraStats(
		int & XSize,							// Number of rows in physical CCD array (no binning)
		int & YSize, 							// Number of columns in physical CCD array (no binning)
		int & MaxBinFactorX,					// Maximum binning factor allowed on X axis
		int & MaxBinFactorY, 					// Maximum binning factor allowed on Y axis
		PBOOL & DifferentAxes, 					// Return true if X and Y binning factors can be different
		PBOOL & HasShutter, 						// Return true if camera has a shutter
		PBOOL & HasCoolerControl, 				// Return true if camera has a controllable cooler setpoint
		PBOOL & PowerOfTwoBinning, 				// Return true if user can select 1/2/4/8/16 (max) binning only
		PBOOL & HasGuiderRelays					// Return true if camera has built-in autoguider output relays
		);

	// GetCameraState returns the current camera status
	void GetCameraState ( 
		CameraState &State,						// Camera state; lower enum values are higher priority
		PBOOL &ShutterOpen						// Return true if shutter is currently open
		);

	// ActivateRelay turns on an autoguider relay output; only one axis is actuated at a time
	int ActivateRelay ( 
		int X,									// Activate X axis for T = abs ( X/100 ) seconds, direction based on sign
		int Y									// Activate Y axis for T = abs ( X/100 ) seconds, direction based on sign
		);

	// IsRelayDone returns true if the autoguider relay is inactive, FALSE if active
	PBOOL IsRelayDone();

	// OpenFilterWheel opens a filter wheel attached to the camera
	int OpenFilterWheel(
		int Param[ NUM_FPARAM ]					// Parameters specified in 
		);

	// CloseFilterWheel disconnects the filter wheel attached to the camera
	void CloseFilterWheel();

	// SetFilterWheel sets the filter wheel position 
	int SetFilterWheel ( 
		int Position							// 0-based index of filter wheel position
		);

	// IsFilterWheelMoving returns true if the filter wheel is moving, FALSE if stationary
	// Note:  Some cameras only move their filters when a StartExposure command is issued; in that case,
	// the GetCameraState command must return CS_FILTERWHEELMOVING until the wheel has stopped
	PBOOL IsFilterWheelMoving();


	// Additional members not forming part of interface
	
	// Simulate a star image
	void MakeStar ( unsigned short *Buffer, double CenterX, double CenterY, double Size ); 

};

#endif // !defined(AFX_DUMMYCCD_H__FF369648_87BA_11D2_AA9C_444553540000__INCLUDED_)
